/*
 *                (c) Copyright Macdonald Associates Ltd. 1983--2008
 *               All rights reserved. All trade marks respected.
 *
 * This software is the exclusive property of Macdonald Associates Ltd.
 * It is furnished under license and may only be used or copied in accordance
 * with the terms of such license.
 * This software is subject to change without notice and no information
 * contained in it should be construed as a commitment by Macdonald Associates.
 * Macdonald Associates cannot accept any responsibility, financial or
 * otherwise, for any consequences arising from the use of this software except
 * as otherwise stated in the terms of the license.
 *
 */

package com.macd.test;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.logging.Level;
import java.util.logging.LogManager;



/** Logging facility.
 *
 *  An instance of this class can be used for logging into a logfile.
 *  The format of the log entries is as follows:
 *  <ul>
 *  <li> date and time
 *  <li> loglevel (one of "info", "warn", "error")
 *  <li> component name
 *  <li> logmessage
 *  <li> (optionally) throwable stack trace<br>
 *       The throwable stack trace is printed in continuation lines,
 *       see example below.
 *  </ul>
 *
 *  Logfile example:
 *  <blockquote><tt><font size=-2>
 *  [Tue Nov 07 11:50:43 CET 2000] [info ] [SWXTrade] read reply: Reply_Code=OK, Local_Trade_Id=3000, Vers_No=1<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [error] [SWXTrade] read reply: Local_Trade_Id is invalid<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [info ] [SWXTrade] processed something<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [warn ] [SWXTrade] some uncritical warning message<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [info ] [SWXTrade] processed something<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [error] [SWXTrade] failed to connect to FOO ressource: caught exception:<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [error] [SWXTrade] ....java.io.IOException: FOO is not ready - sorry.<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [error] [SWXTrade] ....	at com.macd.xConnect.SWXTrade.doBar(SWXTrade.java:233)<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [error] [SWXTrade] ....	at com.fooinc.FooClass.invoke(FooClass:999)<br>
 *  [Tue Nov 07 11:50:43 CET 2000] [error] [SWXTrade] ....	at java.lang.reflect.Method.invoke(Native Method)<br>
 *  </font></tt></blockquote>
 *
 *  <p>
 *  <b>Note:</b>
 *  The logging facility is not implemented as a service since this would be to
 *  much overhead at this time. This means that there is no synchronization of
 *  threads using their own Logger instance and writing to the same logfile. The logfile
 *  is therefore an unprotected shared ressource. We take the (minimal) risk that
 *  this could result in messages from two (or more) threads beeing intermixed.
 *  <br>Nevertheless the logger could be protected by a logging facility component
 *  which would have a private Logger instance and give it's clients an synchronized
 *  interface for logging to one common logfile.
 *
 *  <br>
 *  @version <small><pre>
 *    SVN: repository            $URL: svn+ssh://svnserver/Macd-SVN-Repository/all-macd/branches/ALL_MACD_7_7_2_branch/src/com/macd/utils/Logger.java $
 *    SVN: last committed   $Revision: 5802 $
 *    SVN: last committed       $Date: 2009-07-30 16:02:21 +0200 (Thu, 30 Jul 2009) $
 *    SVN: last committed     $Author: joerg $
 *  </pre></small>
 *
 *  @author (C) Macdonald Associates Ltd. All Rights Reserved.
 */
public final class Logger
{



    /** Call Logger as a java program to log a single line.
     *  <blockquote>
     *      Usage: java com.macd.utils.Logger level componentName messages...
     *  </blockquote>
     *  <p>
     *  <pre>
     *      level         One of INFO, WARNING or ERROR
     *      componentName The name of the component to log this line for.
     *      messages...   Concatenate to get the message to log.
     *  </pre>
     */
    public static void main( String[] arguments ) {

        final String USAGE = "java com.macd.utils.Logger"
                           + " { INFO | WARN | ERROR }"
                           + " <componentName>"
                           + " message...";

        if ( arguments.length < 3 ) {
            System.out.println( USAGE );
            return;
        }

        final String level = arguments[ 0 ].toUpperCase();
        final String componentName = arguments[ 1 ];

        /* Concatenate remaining arguments to one message.
         */
        StringBuffer message = new StringBuffer();

        for ( int index = 2; index < arguments.length; index++ ) {
            message.append( arguments[ index ] ).append( " " );
        }

        final Logger logger = new Logger( componentName );

        if ( level.equals(        "INFO" ) ) {
            logger.logInfo( message.toString() );
        } else if ( level.equals( "WARN" ) ) {
            logger.logWarning( message.toString() );
        } else if ( level.equals( "ERROR" ) ) {
            logger.logError( message.toString() );
        } else {
            logger.logWarning( "Unknown debug level '" + level + "' given -- defaulting to ERROR" );
            logger.logError( message.toString() );
        }
    }


    /**This logger is used for the logging of this class itself
     */
    private final static Logger      logger = new Logger( Logger.class );
    private java.util.logging.Logger myLogger;


    /**
     * Create a new Logger instance and specify
     * the name of the component which uses this logger.
     * <p>
     * The componentName is supposed to be the class name or the name of
     * the service which wants to log.
     * <p>
     * This logger is a wrapper for the Log4J logger. All configuration is
     * to be done through "log4j.properties" in the class path.
     *
     * @param componentName  This name will be written into each log entry
     *                       as the name of the component which is logging.
     *                       The componentName must be a non null String.
     */
    public Logger( final String componentName ) {

        if ( componentName == null ) {
            throw new java.lang.IllegalArgumentException( "Logger componentName must not be null" );
        }

        setLogger( componentName );
    }



    /**
     * Create a new Logger instance using the qualified name of the
     * given class.
     *
     * @param clazz
     */
    public Logger( final Class clazz ) {

        this( clazz, null );
    }



    /**
     * Create a new Logger instance using the qualified name of the
     * given class and the given suffix.
     *
     * @param clazz
     * @param suffix
     */
    public Logger( final Class clazz, final String suffix ) {

        final StringBuilder qualifiedName = new StringBuilder( clazz.getPackage().getName() );
        qualifiedName.append( '.' );
        qualifiedName.append( clazz.getSimpleName() );

        if ( null != suffix ) {
            qualifiedName.append( "-" );
            qualifiedName.append( suffix );
        }

        setLogger( qualifiedName.toString() );
    }



    /**
     * Get the logger for this componentName.
     *
     * @param componentName
     */
    private void setLogger( final String componentName )
    {
        this.myLogger = java.util.logging.Logger.getLogger( componentName );
    }



    /** Write the array of arguments to the logfile as an info message */
    public void logArgs( String[] args ) {

        StringBuffer logmessage = new StringBuffer();

        logmessage.append( "Command line Arguments:" );

        for( int i = 0; i < args.length; i++ ) {
            logmessage.append( " '" )
                      .append( args[i] )
                      .append( "' " );
        }

        logInfo( logmessage.toString() );
    }



    /** Write an debug logmessage to the log.  */
    public void logDebug( String logmessage ) {

        myLogger.log( Level.FINE, logmessage );
    }

    /** Write an debug logmessage to the log plus the throwable's stacktrace.  */
    public void logDebug( String logmessage, Throwable throwable ) {

        myLogger.log( Level.FINE, logmessage, throwable );
    }



    /** Write an info logmessage to the log.  */
    public void logInfo( String logmessage ) {

        myLogger.log( Level.INFO, logmessage );
    }

    /** Write an info logmessage to the log plus the throwable's stacktrace.  */
    public void logInfo( String logmessage, Throwable throwable ) {

        myLogger.log( Level.INFO,  logmessage, throwable );
    }



    /** Write a warning logmessage to the log.  */
    public void logWarning( String logmessage ) {

        myLogger.log( Level.WARNING, logmessage );
    }

    /** Write a warning logmessage to the log plus the throwable's stacktrace.  */
    public void logWarning( String logmessage, Throwable throwable ) {

        myLogger.log( Level.WARNING, logmessage, throwable );
    }



    /** Write a error logmessage to the log.  */
    public void logError( String logmessage ) {

        myLogger.log( Level.SEVERE,  logmessage );
    }

    /** Write a error logmessage to the log plus the throwable's stacktrace.  */
    public void logError( String logmessage, Throwable throwable ) {

        myLogger.log( Level.SEVERE, logmessage, throwable );
    }



    public static void configureLogging( final String level ) {

        if ( null != level ) {

            // Check if it's a log level
            if ( level.equalsIgnoreCase( "debug" ) ) {
                configureLoggingFromTemplate( "FINE" );
            } else if ( level.equalsIgnoreCase( "info" ) ) {
                configureLoggingFromTemplate( "INFO" );
            } else if ( level.equals( "Custom File" ) ) {
                final String propertiesFile = System.getProperty( "user.home" ) + File.separator
                                                + "gloxLogging.properties";
                configureLoggingFromFile( propertiesFile);
            }
        }
    }



    private static void configureLoggingFromFile( String propertiesFile ) {

        try {
            logger.logInfo( "Change logging configuration from file " + propertiesFile );
            LogManager.getLogManager().readConfiguration( new FileInputStream( propertiesFile ) );
        } catch ( Exception e ) {
            logger.logError( "Failed to configure logging from file: " + e.getMessage(), e );
        }
    }



    private static void configureLoggingFromTemplate( String level ) {

        try {
            logger.logInfo( "Change logging configuration from template with log level " + level );
            // Change general logging configuration
            // This configuration is based on the default JRE logging configuration with the
            // following things changed:
            // - Change log level for com.macd.* to given level
            // - Change log level for javax.jms.* to given level
            // - Change log level of FileHandler and ConsoleHandler to given level
            String config =   "handlers= java.util.logging.ConsoleHandler\n"
                            + ".level= INFO\n"
                            + "com.macd.level = " + level + "\n"
                            + "javax.jms.level = " + level + "\n"
                            + "java.util.logging.FileHandler.pattern = %h/java%u.log\n"
                            + "java.util.logging.FileHandler.limit = 50000\n"
                            + "java.util.logging.FileHandler.count = 1\n"
                            + "java.util.logging.FileHandler.formatter = java.util.logging.XMLFormatter\n"
                            + "java.util.logging.FileHandler.level = " + level + "\n"
                            + "java.util.logging.ConsoleHandler.level = " + level + "\n"
                            + "java.util.logging.ConsoleHandler.formatter = java.util.logging.SimpleFormatter\n";
            LogManager.getLogManager().readConfiguration( new ByteArrayInputStream( config.getBytes( "ISO-8859-1" ) ) );
        } catch ( Exception e ) {
            logger.logError( "Failed to change log configuration with template: " + e.getMessage(), e );
        }

    }



}
